/*
 * Copyright (c) 2003-2007, Takahide Matsutsuka.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Takahide Matsutsuka."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * A main file for hex2cas.
 */

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "hex2cas.h"

char* inFilename;
char* outFilename;
char archFilename[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
unsigned char mode;


char *changeExt(const char *pathName, const char* ext) {
  char *p;
  char *tail;
  char *changed;
  int len;
  int extlen = strlen(ext);

  for (tail = (char*) pathName; *tail != 0; tail++);
  for (p = tail; p > pathName; p--) {
    if (*p == '.') {
      len = p - pathName;
      changed = (char*) malloc(len + extlen + 2);

      strncpy(changed, inFilename, len + 1);
      strcpy(changed + len + 1, ext);
      return changed;
    }
  }

  len = strlen(pathName);
  changed = (char*) malloc(len + extlen + 2);
  strncpy(changed, inFilename, len);
  *(changed + len) = '.';
  strcpy(changed + len + 1, ext);
  return changed;
}

void analyzeOption(int argc, char **argv) {
  int c;
  int i;
  opterr = 0;
  while ((c = getopt(argc, argv, "o:n:1258rh")) != EOF) {
    switch (c) {
    case '1':
      // 16k
      mode = MODE1;
      break;
    case '2':
      // 32k
      mode = MODE2;
      break;
    case '5':
      // mk2
      mode = MODE5;
      break;
    case '8':
      // mk2
      mode = MODEN88;
      break;
    case 'r':
      mode = MODEROM;
      break;
    case 'o':
      outFilename = optarg;
      break;
    case 'n':
      for (i = 0; i < 6; i++) {
        if (optarg[i] == 0) {
          break;
      	}
      	archFilename[i] = optarg[i];
      }
      break;
    case 'h':
      mode = MODE_QUIT;
      printf("Options:\n");
      printf(" -o <output file name>\n");
      printf(" -n <cassette file name>\n");
      printf(" -1 mode 1 (PC-6001/16kB)\n");
      printf(" -2 mode 2 (PC-6001/32kB)\n");
      printf(" -5 mode 5 (PC-6001mkII)\n");
      printf(" -r ROM    (PC-6001)\n");
      printf(" -h print this help\n");
    
      return;
    default:
      mode = MODE_QUIT;
      printf("unknown option:%c\n", optopt);
      return;
    }
  }

  return;
}


int main(int argc, char **argv) {
  inFilename = NULL;
  outFilename = NULL;
  mode = 2;

  printf("%s : Convert Intel HEX file to PC-6001 CAS/ROM file version 1.2.0\n", argv[0]);
  printf("Copyright (C) 2003-2007 Takahide Matsutsuka <markn@markn.org>\n");

  analyzeOption(argc, argv);

  inFilename = argv[optind];
  optind++;
  if (optind < argc) {
    analyzeOption(argc, argv);
  }
  if (optind < argc) {
    printf("unknown argument: ");
    for(; optind < argc; optind++){
      printf("%s ", argv[optind]);
    }
    printf("\n");
    return 1;
  }
  if (mode == MODE_QUIT) {
    return 1;
  }
  if (!inFilename) {
    printf("no input specified\n");
    return 1;
  }

  if (outFilename == NULL) {
    outFilename = changeExt(inFilename, "cas");
  }

  printf("Generating file: %s\n", outFilename);
  if (hex2mem(inFilename) == 0) {
    return mem2cas(outFilename, archFilename, mode);
  }
  return -1;
}
